@@ -151,7 +151,7 @@ def main():
     :return: the action output
     """
     # environment variables - get while running the docker
-    region_env = os.environ.get('AWS_DEFAULT_REGION', 'us-east-1')
+    region_env = os.environ.get('AWS_DEFAULT_REGION', 'us-east-2')
     dry_run = os.environ.get('dry_run', 'yes')
 
     account = os.environ.get('account', '')
@@ -177,7 +177,7 @@ def main():
                       'policy': policy,
                       }
         elk_uploader = ESUploader(**input_data)
-        elk_uploader.upload_to_es()
+        elk_uploader.upload_to_es(account=account)
     # 2. POLICY
     else:
         if not policy:
